@@ -20,6 +20,7 @@ import PIL.Image as Image
 from typing import List, Dict, Tuple
 from PIL import ImageDraw, ImageFont
 from math import cos, sin
+from pybaseutils import file_utils
 from pybaseutils.coords_utils import *
 from pybaseutils.transforms import affine_transform
 
@@ -178,7 +179,7 @@ def show_image(title, rgb_image):
     plt.show()
 
 
-def cv_show_image(title, image, use_rgb=True, delay=0):
+def cv_show_image(title, image, use_rgb=False, delay=0):
     """
     调用OpenCV显示RGB图片
     :param title: 图像标题
@@ -220,7 +221,7 @@ def resize_image_like(image_list, dst_img, is_rgb=False):
     return image_list
 
 
-def image_hstack(images, split_line=False, is_rgb=False):
+def image_hstack(images, split_line=False, is_rgb=False, texts=[]):
     """图像左右拼接"""
     dst_images = resize_image_like(image_list=images, dst_img=images[0], is_rgb=is_rgb)
     dst_images = np.hstack(dst_images)
@@ -234,10 +235,12 @@ def image_hstack(images, split_line=False, is_rgb=False):
             p1 = (i * x, 0)
             p2 = (i * x, y)
             dst_images = cv2.line(dst_images, p1, p2, color=(255, 0, 0), thickness=2)
+            if texts: dst_images = draw_text(dst_images, point=(i * x + 10, 30), color=(0, 0, 255),
+                                             fontScale=1.0, thickness=2, text=texts[i], drawType="simple")
     return dst_images
 
 
-def image_vstack(images, split_line=False, is_rgb=False):
+def image_vstack(images, split_line=False, is_rgb=False, texts=[]):
     """图像上下拼接"""
     dst_images = resize_image_like(image_list=images, dst_img=images[0], is_rgb=is_rgb)
     dst_images = np.vstack(dst_images)
@@ -251,6 +254,8 @@ def image_vstack(images, split_line=False, is_rgb=False):
             p1 = (0, i * y)
             p2 = (x, i * y)
             dst_images = cv2.line(dst_images, p1, p2, color=(255, 0, 0), thickness=2)
+            if texts: dst_images = draw_text(dst_images, point=(10, i * y + 30),
+                                             fontScale=1.0, thickness=2, text=texts[i], drawType="simple")
     return dst_images
 
 
@@ -611,7 +616,7 @@ def resize_image_padding(image, size: Tuple, use_length=True, color=(0, 0, 0), i
     return image
 
 
-def resize_image(image, size: Tuple[int, int], interpolation=cv2.INTER_LINEAR):
+def resize_image(image, size: Tuple, interpolation=cv2.INTER_LINEAR):
     """
     tf.image.resize_images(images,size),images=[batch, height, width, channels],size=(new_height, new_width)
     cv2.resize(image, dsize=(width, height)),与image.shape相反
@@ -850,6 +855,20 @@ def scale_rect(orig_rect, orig_shape, dest_shape):
     return dest_rect
 
 
+def scale_box(orig_box, orig_shape, dest_shape):
+    """
+    对图像进行缩放时，对应的rectangle也要进行缩放
+    :param orig_box: 原始图像的box=[xmin,ymin,xmax,ymax]
+    :param orig_shape: 原始图像的维度shape=[h,w]
+    :param dest_shape: 缩放后图像的维度shape=[h,w]
+    :return: 经过缩放后的rectangle
+    """
+    r = (dest_shape[1] / orig_shape[1], dest_shape[0] / orig_shape[0])
+    dest_box = [int(orig_box[0] * r[0]), int(orig_box[1] * r[1]),
+                int(orig_box[2] * r[0]), int(orig_box[3] * r[1])]
+    return dest_box
+
+
 def get_rect_intersection(rec1, rec2):
     """
     计算两个rect的交集坐标
@@ -902,7 +921,7 @@ def draw_image_boxes(bgr_image, boxes_list, color=(0, 0, 255), thickness=1):
     return bgr_image
 
 
-def show_image_rects(title, image, rect_list, color=(0, 0, 255), delay=0):
+def show_image_rects(title, image, rect_list, color=(0, 0, 255), delay=0, use_rgb=False):
     """
     :param title:
     :param image:
@@ -910,11 +929,11 @@ def show_image_rects(title, image, rect_list, color=(0, 0, 255), delay=0):
     :return:
     """
     image = draw_image_rects(image.copy(), rect_list, color)
-    cv_show_image(title, image, delay=delay)
+    cv_show_image(title, image, delay=delay, use_rgb=use_rgb)
     return image
 
 
-def show_image_boxes(title, image, boxes_list, color=(0, 0, 255), delay=0):
+def show_image_boxes(title, image, boxes_list, color=(0, 0, 255), delay=0, use_rgb=False):
     """
     :param title:
     :param image:
@@ -922,7 +941,7 @@ def show_image_boxes(title, image, boxes_list, color=(0, 0, 255), delay=0):
     :return:
     """
     image = draw_image_boxes(image, boxes_list, color)
-    cv_show_image(title, image, delay=delay)
+    cv_show_image(title, image, delay=delay, use_rgb=use_rgb)
     return image
 
 
@@ -1164,7 +1183,7 @@ def custom_bbox_line(image, bbox, color, name, thickness=2, fontScale=0.5, drawT
                       (text_loc[0] + text_size[0], text_loc[1] + text_size[1]), color, -1)
         # draw score value
         cv2.putText(image, str(name), (text_loc[0], text_loc[1] + baseline), cv2.FONT_HERSHEY_SIMPLEX, fontScale,
-                    (255, 255, 255), thickness, 4)
+                    (255, 255, 255), thickness)
     return image
 
 
@@ -1236,7 +1255,7 @@ def show_landmark(title, image, landmarks_list, vis_id=False, delay=0):
     return image
 
 
-def draw_points_text(image, points, texts=None, color=(255, 0, 0), thickness=1, drawType="simple"):
+def draw_points_text(image, points, texts=None, color=(255, 0, 0), fontScale=0.5, thickness=1, drawType="simple"):
     """
     :param image:
     :param points:
@@ -1250,11 +1269,11 @@ def draw_points_text(image, points, texts=None, color=(255, 0, 0), thickness=1,
     for point, text in zip(points, texts):
         point = (int(point[0]), int(point[1]))
         cv2.circle(image, point, thickness * 2, color, -1)
-        draw_text(image, point, text, bg_color=color, thickness=thickness, drawType=drawType)
+        draw_text(image, point, text, color=color, fontScale=fontScale, thickness=thickness, drawType=drawType)
     return image
 
 
-def draw_text(image, point, text, bg_color=(255, 0, 0), thickness=5, drawType="custom"):
+def draw_text(image, point, text, color=(255, 0, 0), fontScale=0.5, thickness=5, drawType="custom"):
     """
     :param image:
     :param point:
@@ -1262,7 +1281,6 @@ def draw_text(image, point, text, bg_color=(255, 0, 0), thickness=5, drawType="c
     :param drawType: custom or simple
     :return:
     """
-    fontScale = 0.5
     text_thickness = 1
     fontFace = cv2.FONT_HERSHEY_SIMPLEX
     # fontFace=cv2.FONT_HERSHEY_SIMPLEX
@@ -1270,12 +1288,12 @@ def draw_text(image, point, text, bg_color=(255, 0, 0), thickness=5, drawType="c
         text_size, baseline = cv2.getTextSize(str(text), fontFace, fontScale, thickness)
         text_loc = (point[0], point[1] + text_size[1])
         cv2.rectangle(image, (text_loc[0] - 2 // 2, text_loc[1] - 2 - baseline),
-                      (text_loc[0] + text_size[0], text_loc[1] + text_size[1]), color=bg_color, thickness=thickness)
+                      (text_loc[0] + text_size[0], text_loc[1] + text_size[1]), color=color, thickness=thickness)
         # draw score value
         cv2.putText(image, str(text), (text_loc[0], text_loc[1] + baseline), fontFace, fontScale,
                     (255, 255, 255), text_thickness, 2)
     elif drawType == "simple":
-        cv2.putText(image, str(text), point, fontFace, fontScale, color=bg_color, thickness=thickness)
+        cv2.putText(image, str(text), point, fontFace, fontScale, color=color, thickness=thickness)
     return image
 
 
@@ -1368,7 +1386,8 @@ def draw_image_points_lines(image,
                             points,
                             pointline=[],
                             texts=None,
-                            circle_color=(0, 255, 0),
+                            fontScale=0.5,
+                            circle_color=(255, 0, 0),
                             line_color=(0, 0, 255),
                             thickness=2):
     """
@@ -1390,6 +1409,7 @@ def draw_image_points_lines(image,
     image = draw_points_text(image, points,
                              texts=texts,
                              color=circle_color,
+                             fontScale=fontScale,
                              thickness=thickness_,
                              drawType="simple")
     return image
@@ -2158,6 +2178,17 @@ def get_contours_iou(contour1, contour2, image_size: Tuple = None, plot=False):
     return contours, iou
 
 
+def get_image_mask(image: np.ndarray, inv):
+    """获得图像的mask"""
+    if image.ndim == 3:
+        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
+    if inv:
+        ret, mask = cv2.threshold(image, 0, 255, cv2.THRESH_BINARY_INV | cv2.THRESH_OTSU)
+    else:
+        ret, mask = cv2.threshold(image, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
+    return mask
+
+
 def get_mask_iou(mask1, mask2, binarize=True):
     """
     计算两个Mask的IOU
@@ -2577,7 +2608,15 @@ def frames2gif_by_pil(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=False)
                        optimize=False, loop=loop)
 
 
-def image_file_list2gif(file_list, size=None, gif_file="test.gif", fps=4, loop=0, use_pil=True):
+def image_dir2gif(image_dir, size=None, gif_file=None, interval=1, fps=4, loop=0, padding=True, use_pil=True):
+    if not gif_file: gif_file = os.path.join(os.path.dirname(image_dir), os.path.basename(image_dir) + ".gif")
+    file_list = file_utils.get_images_list(image_dir)
+    image_file2gif(file_list, size=size, gif_file=gif_file, interval=interval, fps=fps, loop=loop,
+                   padding=padding, use_pil=use_pil)
+
+
+def image_file2gif(file_list, size=None, gif_file="test.gif", interval=1, fps=4, loop=0,
+                   padding=True, use_pil=True):
     """
     pip install imageio
     uri：合成后的gif动图的名字，可以随意更改。
@@ -2597,14 +2636,15 @@ def image_file_list2gif(file_list, size=None, gif_file="test.gif", fps=4, loop=0
     :return:
     """
     frames = []
-    for file in file_list:
-        bgr = cv2.imread(file)
-        image = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
-        if size:
-            image = resize_image_padding(image, size=size)
-        else:
-            image = resize_image(image, size=size)
-        frames.append(image)
+    for count, file in enumerate(file_list):
+        if count % interval == 0:
+            bgr = cv2.imread(file)
+            image = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
+            if padding:
+                image = resize_image_padding(image, size=size)
+            else:
+                image = resize_image(image, size=size)
+            frames.append(image)
     if use_pil:
         frames2gif_by_pil(frames, gif_file, fps=fps, loop=loop)
     else:
@@ -2657,16 +2697,16 @@ class CVVideo():
     def __init__(self):
         pass
 
-    def start_capture(self, video_path, save_video=None, detect_freq=1):
+    def start_capture(self, video_file, save_video=None, detect_freq=1):
         """
         start capture video
-        :param video_path: *.avi,*.mp4,...or camera id
+        :param video_file: *.avi,*.mp4,...or camera id
         :param save_video: *.avi
         :param detect_freq:
         :return:
         """
         # cv2.moveWindow("test", 1000, 100)
-        video_cap = get_video_capture(video_path)
+        video_cap = get_video_capture(video_file)
         width, height, numFrames, fps = get_video_info(video_cap)
         if save_video:
             self.video_writer = get_video_writer(save_video, width, height, fps)
